@@ -29,6 +29,14 @@ FQDN = "fqdn"
 def parseargs() -> argparse.ArgumentParser:
     """Parse arguments"""
     parser = worker.parseargs("PDNS enrichment")
+
+    parser.add_argument(
+        "--no-tlp-access-mode",
+        action="store_true",
+        help="Do not set access mode from TLP on document. If not set, facts will "
+        + "have access-mode=Public for tlp=white/green otherwise access-mode=RoleBased",
+    )
+
     parser.add_argument(
         "--pdns-baseurl",
         dest="pdns_baseurl",
@@ -130,6 +138,7 @@ def process(
     output_format: Text = "json",
     batch_size: int = 1000,
     limit: int = 0,
+    no_tlp_access_mode: bool = False,
 ) -> None:
     """Read queries from stdin, resolve each one through passivedns
     printing generic_uploader data to stdout"""
@@ -151,18 +160,29 @@ def process(
         ):
             rrtype = row["rrtype"]
 
+            if no_tlp_access_mode:
+                # Use Default Access Mode
+                access_mode = api.config.access_mode
+            else:
+                # Set Access Mode based on TLP
+                access_mode = (
+                    "Public" if row.get("tlp") in ("green", "white") else "RoleBased"
+                )
+
             i += 1
             if limit == i:
                 if kind(query) in (IPv4, IPv6):
                     act.api.helpers.handle_fact(
-                        api.fact("excessive", "resolvesTo").source(
-                            *act.api.helpers.ip_obj(row["answer"])
-                        ),
+                        api.fact(
+                            "excessive", "resolvesTo", access_mode=access_mode
+                        ).source(*act.api.helpers.ip_obj(row["answer"])),
                         output_format=output_format,
                     )
                 else:
                     act.api.helpers.handle_fact(
-                        api.fact("excessive", "resolvesTo").source("fqdn", query),
+                        api.fact(
+                            "excessive", "resolvesTo", access_mode=access_mode
+                        ).source("fqdn", query),
                         output_format=output_format,
                     )
 
@@ -172,7 +192,7 @@ def process(
 
             if rrtype in ("a", "aaaa"):
                 act.api.helpers.handle_fact(
-                    api.fact("resolvesTo")
+                    api.fact("resolvesTo", access_mode=access_mode)
                     .source("fqdn", row["query"])
                     .destination(*act.api.helpers.ip_obj(row["answer"])),
                     output_format=output_format,
@@ -180,7 +200,7 @@ def process(
 
             elif rrtype == "cname":
                 act.api.helpers.handle_fact(
-                    api.fact("resolvesTo")
+                    api.fact("resolvesTo", access_mode=access_mode)
                     .source("fqdn", row["query"])
                     .destination("fqdn", row["answer"]),
                     output_format=output_format,
@@ -207,6 +227,7 @@ def main() -> None:
         args.output_format,
         args.pdns_batch_size,
         args.limit,
+        args.no_tlp_access_mode,
     )
 
 
